
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "cube.h"
#include "dict.h"
#include "boggle.h"

typedef struct tv {
    long        tv_sec;     /* seconds (XXX should be time_t) */
    long		tv_usec;    /* and microseconds */
} beartv ;



int time_diff (beartv time1, beartv time2){
	int diff;
	int i;
	diff=time2.tv_usec;
	for (i=0; i < time2.tv_sec - time1.tv_sec; i++) 
		diff+=1000000;
	diff-=time1.tv_usec;
	return diff;

}



int main (int argc, char** argv) {
	char* cube;
	Dict dict, comp;
	beartv start, insertion, sort, solve_start, solve_end;
	int i, time_constant, z=0;
	FILE *file;

	dict = createDict () ;
	cube = malloc (20);

	if ( argc > 1 )
		file = fopen(argv[1], "r");
	else
		file = stdin;

	if (!file) {
		printf("Error opening %s", argv[1]);
		return -20;
	}

	for (i=0; i < 16; i++) {
		while (!isalpha(cube[i] = fgetc(file)));
	}

	gettimeofday ( &start, 0 );
	gettimeofday ( &sort , 0);
	time_constant = time_diff(start, sort);

	gettimeofday ( &start , 0);
	insertFile ( dict, "dictionary.dat" );
	gettimeofday ( &insertion, 0 );

	sortDict (dict);
	gettimeofday ( &sort , 0);
	radixDict(dict, 0, 0); 

	for (i=3; i>=0 ; i--)
	    printf("\t%c %c %c %c\n", cube[i*4],     cube[1+(i*4)],
	                              cube[(i*4)+2], cube[(i*4)+3] );

	solveCube (cube, dict);

	gettimeofday ( &solve_start, 0 );
	comp=solveCube (cube, dict);
	gettimeofday ( &solve_end, 0 );

	sortDict (comp);
	printDict(comp);

	printf("\n\n\n\nFound %d words\n", SizeOfDict(comp) );
	printf("insertFile took %10d     mS\n", (time_diff(start, insertion) - time_constant)/1000);
	printf("sortDict took   %10d     mS\n", (time_diff(insertion, sort)    - time_constant)/1000);
	printf("solveCube took  %10d.%3.3d mS (+- .010 mS)\n", (time_diff(solve_start, solve_end)   - time_constant)/1000,
										   (time_diff(solve_start, solve_end)   - time_constant)%1000);
	fclose(file);
}

