/* Cube ADT
 * 
 * The code simulated boogle dice. The process is as follows
 *
 *  1) Read cubes from a file, format is ;
 *      16 lines total, where each line corresponds to a six sided die
 *      the line format is 6 litters corresponding to the die's six sides.
 *  2) Randomize the placement, and roll each die
 *  3) Return a 4x4 matrix consisting of the chosen letters for Boggle
 *
 */

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include "dict.h"

static char* cube=0;

/* returns a 4x4 cube consisting of pseudo randomized letters. Read input
 * from file given by filename
 */
char* createCube(char* filename) {
	FILE *file;
	int i=0, j, number, selection;
	char die[10], head;

	if (!cube)
		cube=malloc(20);

	file = fopen (filename, "r");
	if (!file) {
		printf("Your dice file is like totally lame, dude\n");
		exit(1);
	}
	srandom((random() & 0x7fff) + (time(NULL) & 0xfff) * 0x712);
	for (i=0; i < 16; i++)
		cube[i]=0;

	for (i=0; i < 16; i++) {
		fgets ( die, 10, file );
		head = die [ random() % 6 ];
		/* Do random die placement by doing rand 16, then rand 15, then ...
		 * each time, we subtract out the just placed die, so for instance if
		 * we choose a 8, and we already picked a 2 and 3, add 2 to 8.
		 */
		 selection = random () % ( 16 - i);
		 for (j=0; j < selection; j++) 
			if ( cube[j] )
				selection++;
		while (cube[selection]) /* fix number and number again problem */
			selection++;
		cube[selection] = head;

	}

	fclose (file);
	return cube;

}

/* printCube displays the most recently created cube to stdout
 */
void printCube () {
	int i;
	for (i=3; i>=0 ; i--) 
		printf("\t%c %c %c %c\n", cube[i*4],     cube[1+(i*4)], 
								  cube[(i*4)+2], cube[(i*4)+3]); 
}

void doSolveCube(char* cube, Dict dict, Dict results, int start, char* word) {
	char my_letter;
	int len, i, result;

	if ( !cube[start] ) 
		return ;

	len = strlen(word) ;

	word[len] = tolower(cube[start]);
	word[len+1] = 0;

	result = isWord ( dict, word );

	if ( strlen(word) >= 4 )
		if (result & 0x02) // word is an exact match
			if ( !(isWord(results, word) & 0x02) ) //word not a result
				insertWord ( results, word );

	if ( !(result & 0x01) )  {//word is not a prefix
		word[len]=0;
		return ;
	}

	my_letter = cube[start];
	cube[start]=0;

	/* Try all the possible directions  of movement */

	if ( start > 3 ) { /* Do BOTTOM */
		if ( (start & 0x3) ) 
			doSolveCube ( cube, dict, results, start - 5, word );
		doSolveCube ( cube, dict, results, start - 4, word );
		if ( (start & 0x3) != 3 )
			doSolveCube ( cube, dict, results, start - 3, word );
	}

	/* Do Middle */
	if ( (start & 0x3) )
		doSolveCube ( cube, dict, results, start - 1, word );
	if ( (start & 0x3) != 3 )
		doSolveCube ( cube, dict, results, start + 1, word );

	if ( start < 12 )   { /* Do Top */
		if ( (start & 3) ) 
			doSolveCube ( cube, dict, results, start + 3, word );
		doSolveCube ( cube, dict, results, start + 4, word );
		if ( (start & 3) != 3 )
			doSolveCube ( cube, dict, results, start + 5, word );
	}
	


	cube[start] = my_letter;
	word[len]   = 0;
}

Dict solveCube ( char* cube, Dict dict ) {
	Dict matches;
	static char* word=0;
	int i;

	if (!word)
		word=malloc(30);
	bzero ( word, 20);

	matches=createDict();
	for (i=0; i < 16; i++) 
		doSolveCube(cube, dict, matches, i, word );
	return matches;

}

