#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dict.h"

int glob_prefix;

Dict createDict(void) {
	Dict newDict;

	newDict = malloc (32);
	bzero(newDict, 20);

	newDict->max=16;
	newDict->word = malloc ( 16* newDict->max );
	bzero( newDict->word, newDict->max * 16 );

	return newDict;
}

void deleteDict(Dict dict) {
	if (!dict || !dict->word)
		return;
	free ( dict->word );
	free ( dict );
}

void doubleDict(Dict dict) {
	void* old;
	dict->max = dict->max << 1;
	old = dict->word;
	dict->word = malloc ( 16*dict->max );
	bzero(dict->word, 16*dict->max );
	memcpy ( dict->word, old, dict->last << 4 );
}
/* Insert's word into dict dictionary. Sets the internal paramater
 *  'sorted' to false. 
 */
void insertWord(Dict dict, char* word) {
	int i;

	if ( (dict->last+1) >= dict -> max )
		doubleDict ( dict );
	dict->sorted=0;

	dict->last++;

	for (i=0; i < 16; i++) {
		if (!word[i])
			return;
		 *((char*)(dict->word+dict->last*16-16+i)) = tolower ( word[i] );

	}
	return ;

	strncpy (dict->word+(dict->last*16), word, 16);
	dict->last++;

}

/* InsertFile inserts all words in fileName to dict.
 *  File format should be word\nword2\nword3\n, where \n is a new
 *  line.
 */
void insertFile(Dict dict, char* file) {
	FILE *File;
	char* string = malloc (80);
	File = fopen ( file, "r" );
	if (!File) {
		printf("could not open %s, the dictionary file\n", file);
		exit(1);
	}
	while ( fgets ( string, 79, File ) ) {
		string [strlen (string) - 1] = 0;
		insertWord ( dict, string ) ;
	}
}

#define NO_MATCH_GT 10
#define NO_MATCH_LT -2
#define   PREFIX_LT -1 

int wordCompare ( const void* vfoo, const void* vbar ) {
	int i;
	char* foo = (char*) vfoo; 
	char* bar = (char*) vbar; 

	for (i=0; i < 16 ; i++) {
		if ( !foo[i] && !bar[i] )
			return 0; /* MATCH */
		if ( !foo[i] ) {
			glob_prefix=1;
			return PREFIX_LT;
		}
		if ( foo[i] > bar[i])
			return NO_MATCH_GT;
		if ( foo[i] < bar[i] )
			return NO_MATCH_LT;
	}

	return 0; /* 16 letter match */
}


/* Check whether a given set of letters ever appears at the beginning of a word. You might use the dictionary or you define some rules (that will be explained in the document file). */
/* boolean IsPrefix(dictionaryADT dict, string prefix);

/* isWord is a trinary operation ( a concatonation of isPrefix and isWord ). 
 *
 *    isWord does a binary search of the dictionary file if 'sorted' is set 
 *    to true (i.e. SortDict has been called after last InsertWord), other- 
 *    wise, do a linear search.
 * 
 * returns:
 *
 *  zero   if no match    
 *  | 1    if word is a prefix
 *  | 2    if word is a match
 *
 * Bugs:
 *    Should really be call isntWord, to be consistent with isWord funcs.
 */
int isWord(Dict dict, char* string) {
	int i=0, result, prefix=0;

	if (dict->sorted == 2)
		return radixFind(dict, string);

	if ( !dict->sorted ) {

		for (i=0; i < dict->last; i++) {
			result = wordCompare ( string, dict->word+i*16 );
			if (!result)
				prefix|=0x02;
			if (result == PREFIX_LT)
				prefix|=0x01;
		}

		return prefix;

	} else {
		glob_prefix=0;
		if (bsearch ( string, dict->word, dict->last, 16, &wordCompare))
			return 0x02 | 0x01; /* assume prefix, since it might be */
		return glob_prefix;
	}



}


/* return the number of words in the dictionary */
int SizeOfDict(Dict dict) {
	return dict->last;
}

/* print the contents of dict */
void printDict(Dict dict) {
	int i;
	char word[20];
	word[17]=word[18]=word[19]=word[16]=0;

	for (i=0; i < dict->last; i++) {
		memcpy ( word, dict->word+(i*16), 16 );
		printf("%s\n", word);
	}
}

/* Set 'sorted' flag to true. Changes look up time from a O(n) operation
 * to a O( log(n) ) operation. Users of the ADT should take care to leave
 * dict in a sorted state if dict is used with large n.
 */
void sortDict(Dict dict) {
	qsort ( dict->word, dict->last, 16, &wordCompare );
	dict->sorted=1;
}

/* Counts the number of occurances of prefix in dict
 *
 * returns:  0 - prefix doesn't match any words
 *           1 - prefix matches exactly one word 
 *           2 - prefix matches more than two words
 *
 * Note:     the difference between this function and isWord is
 *           that on a succesful match, we still look to see if 
 *           prefix is a prefix to any other word.
 */
int wordCount (char* prefix, Dict dict) {
		char* word;
		glob_prefix=0;
		word = bsearch ( prefix, dict->word, dict->last, 16, &wordCompare);
		if (!word) {
			if (glob_prefix)
				return 2;
			else
				return 0;
		}

		word+=16;
		if (wordCompare(prefix, word) == PREFIX_LT) 
			return 2;
		return 1;


}

/* Create a radix style index, for very fast look up times */
void* radixDict(Dict dict, char* word_full, char* word_pos ) {
	radixType *radix;
	static char word[20];
	int i;
	int first=0;

	radix = malloc (sizeof (radixType));

	if (!word_full) {
		word_full = word_pos = word;
		first=1;
	} else {
		if (isWord (dict, word_full) & 0x02)
			radix->isWord=1;
		else
			radix->isWord=0;
		word_pos++;
	}
	word_pos[0]='a';

	word_pos[1]=-0;



	for (i=*word_pos; i <= 'z'; i++) {
		*word_pos=i;

		#ifndef QUIET
		if (first)
			printf("\rCreating radix index, %2d%c percent complete.", ((i-0x60)*100)/26,'%');
		#endif

		fflush(stdout);
		switch( wordCount ( word_full, dict) ) {
			case 0: 
				radix->letter[i-0x61] = 0;
				break;
			case 1:
				radix->letter[i-0x61] = bsearch 
					( word_full, dict->word, dict->last, 16, &wordCompare) + 1;
				break;
			case 2:
				radix->letter[i-0x61]=radixDict(dict, word_full, word_pos);
				word_pos[1]=0;
				break;
			default:
				printf("Assert: wordCount != 0-2\n");
				exit(1);
		}
	}

	if (first) {
		dict->sorted=2;

		#ifndef QUIET
		printf("\n");
		#endif

	}
	dict->radix=radix;
	return radix;

}

/* radixFind mimics the functionality of isWord, except that it is O(1)
 * instead of O(log(n)) and returns more information about the match.
 * in particular, it differentiates between an exact match that is a 
 * prefix and a exact match that is not a prefix
 *
 * Returns:
 *
 *  zero   if no match    
 *  | 1    if word is a prefix
 *  | 2    if word is a match
 */

int radixFind(Dict dict, char* word) {
	radixType *radix;
	radix = dict->radix;
	if (!word) {printf("Assert radixFind != NULL is null\n"); exit(1);}
	while (1) {

		if (*word) {
			if (radix->letter[*word - 0x61]) {
				if (((int)(radix->letter[*word - 0x61])) & 1)
					return 0x02; /*word is a only match */
				else
					radix = radix->letter[*word - 0x61];
			} else
				return 0; /* no match, TERMINATING CASE */
		} else {
			if (radix->isWord)
				return 0x01 | 0x02; /* word is a prefix and dictionary word */
			else
				return 0x01; /* Word is a prefix */

		} 

		word++;
	}
			

}

#ifdef DICT_TEST
int main() {
	Dict dict;
	dict=createDict();
	insertWord( dict, "AABBCCDDEEFFGGHHIIJJKKLLMMNNOOPPQQRRSSTTUUVV" );
	insertWord( dict, "bored" );
	insertWord( dict, "out" );
	insertWord( dict, "of" );
	insertWord( dict, "sanity" );
	insertWord( dict, ",psycho" );
	insertFile( dict, "/usr/share/dict/words" );
	sortDict ( dict );
	printf("Isword (bored, bogu, lsdkfj) = (%d, %d, %d)\n",
		isWord(dict, "bored"),
		isWord(dict, "bogu"),
		isWord(dict, "lsdkfj") );

}
#endif
