/* Dictionary ADT. 
 *
 *  A set of access routines for storing words up to 16 bytes (128 bits) in
 *  length.
 *
 *  Assumptions:
 *    Null termination for char* is padded to the end of word size.
 */


#ifndef __DICT__DOT__POO__
#define __DICT__DOT__POO__

typedef struct radixStruct {
	int isWord;
	void* letter[26];
} radixType;

typedef struct DictStruct {
	int max;
	int last;
	int sorted;
	radixType* radix;
	void* word;
} DictStruct;


typedef struct DictStruct * Dict;

Dict createDict(void);
void deleteDict(Dict);

/* Insert's word into dict dictionary. Sets the internal paramater
 *  'sorted' to false.
 */
void insertWord(Dict, char*);

/* InsertFile inserts all words in fileName to dict.
 *  File format should be word\nword2\nword3\n, where \n is a new
 *  line.
 */
void insertFile(Dict, char* );


/* Check whether a given set of letters ever appears at the beginning of a word. You might use the dictionary or you define some rules (that will be explained in the document file). */
/* boolean IsPrefix(dictionaryADT dict, string prefix);

/* isWord is a trinary operation ( a concatonation of isPrefix and isWord ). 
 *
 *    isWord does a binary search of the dictionary file if 'sorted' is set 
 *    to true (i.e. SortDict has been called after last InsertWord), other-
 *    wise, do a linear search.
 * 
 * returns:
 *   positive if word is a prefix to a word in the dictionary
 *   zero     if word matches a word in the dictionary
 *   negative if word is not a prefix and is not in the dictionary file 
 */
int isWord(Dict, char*);


/* return the number of words in the dictionary */
int sizeOfDict(Dict);

/* print the contents of dict */
void printDict(Dict);

/* Set 'sorted' flag to true. Changes look up time from a O(n) operation
 * to a O( log(n) ) operation. Users of the ADT should take care to leave
 * dict in a sorted state if dict is used with large n.
 */
void sortDict(Dict);

/* Create a radix style index of the dictionary for O(1) look ups...
 * Has significant memory overhead. Sets internal sorted flag to 2, 
 * meaning do radix lookup.
 */
void* radixDict(Dict, char*, char*);


#endif
